do

local TTT_Path = import('/lua/game.lua').TTT_ModPath
local Config = import(TTT_Path..'/lua/TTT_Config.lua')
local Votes = {}
local NumVotingPlayers = 0
local WaitTime = {}
local VoteOver = false
local NumVotesIn = 0

#### VOTE STUFF ####

local function PlayerVoted(data)

    local playerId = data.From
    local playerName = data.Args.playerName
    local vote = data.Args.option

    if NumVotingPlayers > 0 and ArmyBrains[playerId].BrainType == 'Human' and not ArmyIsOutOfGame(playerId) then

        if Votes[vote] then
            Votes[vote] = Votes[vote] + 1
            NumVotesIn = NumVotesIn + 1
            Sync.TTT_NumVotesIn = NumVotesIn
            if NumVotesIn >= NumVotingPlayers or Votes[vote] > (NumVotingPlayers / 2) then
                EndVote()
            end
        else
            # unknown or illegal vote
            Sync.TTT_ShowVoteUI = true
        end
    end
end

local function StartVote()

    VoteOver = false

    # current number of votes for each vote option
    Votes = {}
    for k, v in Config.Vote.VoteOptions do
        Votes[k] = 0
    end

    # send this stuff to the UI layer
    Sync.TTT_VoteOptions = Config.Vote.VoteOptions
    Sync.TTT_Votes = Votes
    Sync.TTT_VoteTime = Config.Vote.VotingTime

    # using the query functionality to allow the UI send it's vote back to us
    import('/lua/SimPlayerQuery.lua').AddQueryListener("TimeOverTechIntervalVote", PlayerVoted)

    # show voting UI for all relevant players
    if ArmyBrains[GetFocusArmy()].BrainType == 'Human' and not ArmyIsOutOfGame(GetFocusArmy()) then
        Sync.TTT_ShowVoteUI = true
    end
    for army, brain in ArmyBrains do
        if brain.BrainType == 'Human' and not ArmyIsOutOfGame(army) then
            NumVotingPlayers = NumVotingPlayers + 1
        end
    end
end

function EndVote()

    if not VoteOver then

        VoteOver = true

        # hide vote UI for all players
        for army, brain in ArmyBrains do
            if brain.BrainType == 'Human' then
                Sync.TTT_ShowVoteUI = false
            end
        end

        # get most voted option
        local winningOption = 0
        for option, numVotes in Votes do
            if numVotes > Votes[winningOption] or not Votes[winningOption] then
                winningOption = option
            end
        end

        # set wait times according to most voted option
        for k, tech in Config.Tech.TechLevels do
            if not tech.AlwaysAvailable then
                SetWaitTime(k, Config.Vote.VoteOptions[winningOption] )
            end
        end

        # tell players what option won the voting
        Sync.TTT_WinningOption = winningOption
        if ArmyBrains[GetFocusArmy()].BrainType == 'Human' or GetFocusArmy() == -1 then
            Sync.TTT_AnnounceVoteOver = true
        end
    end

end

#### TECH OVER TIME STUFF ####

local function SetTechAvailable( tech, notAvailable )
    if not tech.Restricted then
        for army, brain in ArmyBrains do
            if not ArmyIsOutOfGame(army) then
                if notAvailable then
                    AddBuildRestriction(army, ParseEntityCategory(tech.UnitCategory))
                else
                    RemoveBuildRestriction(army, ParseEntityCategory(tech.UnitCategory))
                end
            end
        end
        if tech.Announce and not notAvailable then
            if ArmyBrains[GetFocusArmy()].BrainType == 'Human' or GetFocusArmy() == -1 then
                Sync.TTT_AnnounceNewTechAvailable = tech
            end
        end
    end
end

local function LockAllLockableTech()
    for k, tech in Config.Tech.TechLevels do
        if not tech.AlwaysAvailable then
            SetTechAvailable( tech, true)
        end
    end
end

local function UnlockTechOverTime()
    for k, tech in Config.Tech.TechLevels do
        if not tech.AlwaysAvailable then
            WaitSeconds(WaitTime[k])
            SetTechAvailable(tech)
        end
    end
end

local function CheckUnitRestrictions()

    local restrictedUnits = import('/lua/ui/lobby/restrictedUnitsData.lua').restrictedUnits
    for k, restriction in ScenarioInfo.Options.RestrictedCategories do 
        for l, cat in restrictedUnits[restriction].categories do
            for m, tech in Config.Tech.TechLevels do
                if tech.UnitCategory == cat then
                    Config.Tech.TechLevels[m].Restricted = true
                end
            end
        end
    end
end

function SetWaitTime( key, voteOption )
    local seconds = voteOption.Interval
    if seconds == -1 then
        seconds = Random( voteOption.Min, voteOption.Max )
    end
    WaitTime[key] = seconds
end

function InitUnlockTechOverTime()

    # make sure we don't override unit restrictions
    CheckUnitRestrictions()

    # lock all tech but T1
    LockAllLockableTech()

    # pre-vote time. Don't want to distract users in the first minute
    WaitSeconds(Config.Vote.PreVotingTime)

    # start vote
    StartVote()

    # wait for voting to complete
    WaitSeconds(Config.Vote.VotingTime)

    # end vote if it wasn't
    EndVote()

    # adjust T2 wait time because of the 3 minute vote time wait    had to go for this ugly sollution :-(
    for k, v in WaitTime do
        WaitTime[k] = math.max( 0, math.floor( WaitTime[k] - GetGameTimeSeconds()) )
        break
    end

    # start the thread
    ForkThread(UnlockTechOverTime)
end


end